/*
 *  Copyright
 */

#ifndef __CONFIG_SWAP_H
#define __CONFIG_SWAP_H

// ==========================================================================

#include "KineoWorks2/kwsConfig.h"

using namespace std ;

// ==========================================================================

/**
  \brief   Swap an HPP Config of HRP2 to an openHRP config of HRP2 because the kinematic chain description are different
 
     - OpenHRP : RLEG(6) LLEG(6) CHEST(2) HEAD(2) RARM(7) LARM(7) RHAND(5) LHAND(5)

     - Kineo   : WAIST(6) RLEG(6) LLEG(6) CHEST(2) HEAD(2) RARM(7) RHAND(5) LARM(7) LHAND(5)

  \param  hppConfig of HRP2 kwsconfig of 46 dof
  \param  openHRPConfig return the corresponding configuration in openHRPConfig vector of 40 double
*/ 

void toOpenHRPConfig (CkwsConfigShPtr hppConfig, vector<double> &openHRPConfig) ;


/**
  \brief   Swap an openHRP config of HRP2 to an HPP config of HRP2 because the kinematic chain description are different
 
     - OpenHRP : RLEG(6) LLEG(6) CHEST(2) HEAD(2) RARM(7) LARM(7) RHAND(5) LHAND(5)

     - Kineo   : WAIST(6) RLEG(6) LLEG(6) CHEST(2) HEAD(2) RARM(7) RHAND(5) LARM(7) LHAND(5)

  \param  openHRPConfig of HRP2 vector 40 double
  \param  waistPosition position of the waist corresponding to the configuration vector 6 double (X, Y, Z, RX, RY, RZ)
  \param  hppConfig return the corresponding configuration in hppConfig of 46 dof
*/ 
void toHppConfig (vector<double> openHRPConfig, vector<double> waistPosition, CkwsConfigShPtr &hppConfig ) ;

/**
   \brief convert roll-pitch-yaw angles(ZYX Euler angles) to Kineo rotation angles
   \param inR roll
   \param inP pitch
   \param inY yaw
   \param outX rotation around X
   \param outY rotation around new Y
   \param outZ rotation around new Z
*/
void toHppRotation(const double inR, const double inP, const double inY,
		   double &outX, double &outY, double &outZ);

#endif
