#include <iostream>
#include "hppOpenHRP/parserOpenHRPKineo.h"

// ==============================================================================

void CparserOpenHRPKineo::setPolyColor(CkppKCDPolyhedronShPtr hppPolyhedron, const ModelInfo::shortSeq *colorIndexes, 
				       const std::vector < CkppMaterial > materialVector) {

  unsigned int nbTriangles = colorIndexes->length();
  unsigned int colorIndex, colorIndex_prev;
  unsigned int startTriRank=0, crntMaterialIndex;
  for(unsigned int i=0; i<nbTriangles; i++){
    colorIndex = (*colorIndexes)[i];

    if(i==0){ // first 
      startTriRank = i;
      crntMaterialIndex = colorIndex;
    }
    else if(colorIndex != colorIndex_prev){
      // i is different from previous: set material startTriRank -- i-1
      hppPolyhedron->setMaterial(startTriRank, i-1, materialVector[crntMaterialIndex]);
      // debug
      // cerr<<"setMaterial ("<<startTriRank<<", "<<i-1<<", matIndex "<<crntMaterialIndex<<")"<<endl;
      startTriRank = i;
      crntMaterialIndex = colorIndex;
    }

    if(i == nbTriangles-1){ // last
      hppPolyhedron->setMaterial(startTriRank, i, materialVector[crntMaterialIndex]);
      // debug
      // cerr<<"setMaterial ("<<startTriRank<<", "<<i<<", matIndex "<<crntMaterialIndex<<")"<<endl;
    }
    else
      colorIndex_prev = colorIndex;
      
  }
}

// ==============================================================================

void CparserOpenHRPKineo::getPositionMatrix(const ValueMatrix *relPos, 
					    const ValueMatrix *relAttitude,
					    CkitMat4& kwsRelPosMat){
  double vKwsRelPosMat[4][4];
  for (unsigned int i=0; i<3; i++) {
    vKwsRelPosMat[i][3] = (*relPos)[i];
    for (unsigned int j=0; j<3; j++) {
      vKwsRelPosMat[i][j] = (*relAttitude)[j+3*i];
    }
  }
  vKwsRelPosMat[3][0] = vKwsRelPosMat[3][1] = vKwsRelPosMat[3][2] = 0;
  vKwsRelPosMat[3][3] = 1.0;
    
  kwsRelPosMat = vKwsRelPosMat;
    
}

// ==============================================================================

CkppKCDPolyhedronShPtr CparserOpenHRPKineo::buildHppPolyhedron(const ModelInfo_ptr inOpenHRPModelInfo,
							       const TopenHrpJoint& openHRPObstModel,
							       const char *inName) 
{
  // create empty polyhedron.
  CkppKCDPolyhedronShPtr hppPolyhedron ;
  hppPolyhedron = CkppKCDPolyhedron::create(inName);  
  if (!hppPolyhedron) {
    hppPolyhedron.reset() ;
    return hppPolyhedron;
  }
    
    
  // Set vertices
  unsigned int nbVertices=0, nv;
  do{
      ParserModelObject::floatSeq *vertices;
      vertices = openHRPObstModel->readVertices(NUM_MAX_TRI);
      nv = vertices->length()/3; 
      unsigned int rank;
      for(unsigned int i=0; i<nv; i++){
	  hppPolyhedron->CkcdPolyhedron::addPoint((double)(*vertices)[i*3], 
						  (double)(*vertices)[i*3+1], 
						  (double)(*vertices)[i*3+2],
						  rank);
	  if (rank != i+nbVertices){
	      std::cerr << "CparserOpenHRPKineo::buildHppPolyhedron : mismatch rank("
			<< rank << "<->" << i+nbVertices << ")" << std::endl;
	  }
      }
      nbVertices += nv;
  }while(nv == NUM_MAX_TRI);

  // Set triangles
  unsigned int nbTriangles = 0;
  do{
      ParserModelObject::ulongSeq *triangles;
      triangles = openHRPObstModel->readTrianglesByIndexes(NUM_MAX_TRI);
      nbTriangles = triangles->length()/3;
      unsigned int rank;
      for (int i=0; i<nbTriangles; i++) {
	  if ((*triangles)[i*3]>=nbVertices
	      || (*triangles)[i*3+1]>=nbVertices
	      || (*triangles)[i*3+2]>=nbVertices){
	      std::cerr << "CparserOpenHRPKineo::buildHppPolyhedron : rank of vertex is out of range(nbVertices = " << nbVertices << ", rank = (" 
			<< (*triangles)[i*3] << ", "
			<< (*triangles)[i*3+1] << ", "
			<< (*triangles)[i*3+2] << ")" << std::endl;
	  }
	  hppPolyhedron->addTriangle((*triangles)[i*3], (*triangles)[i*3+1], 
				     (*triangles)[i*3+2], rank);
      }
  }while(nbTriangles == NUM_MAX_TRI);

  

  // Get colors
  std::vector < CkppMaterial > materialVector;
  ModelInfo::floatSeq *diffuseColors;
  diffuseColors = inOpenHRPModelInfo->readDiffuseColors(openHRPObstModel->name());
  int nbDiffuseColors = diffuseColors->length()/3;
  float rgb[3], alpha = 0.4;
  for(unsigned int i=0; (int)i<nbDiffuseColors; i++){

    for(unsigned int j=0; j<3; j++){
      rgb[j] = (*diffuseColors)[3*i+j];
    }
    // create material
    CkppMaterial material;
    material.diffuseColor(CkppColor( rgb[0], rgb[1], rgb[2], alpha ));
    materialVector.push_back(material);
  }

  // put color
  ModelInfo::shortSeq *colorIndexes;
  colorIndexes = inOpenHRPModelInfo->readColorIndexes(openHRPObstModel->name());
  setPolyColor(hppPolyhedron, colorIndexes, materialVector);

  return hppPolyhedron;
};
